/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

  As a special exception, permission is given to link this program
  with any edition of Qt, and distribute the resulting executable,
  without including the source code for Qt in the source distribution.
*/

#include "MabumbiClient.h"
#include "MabumbiInterface.h"
#include "MabumbiSysTray.h"

#include <KDE/KAction>
#include <KDE/KActionCollection>
#include <KDE/KMenu>
#include <KDE/KMenuBar>
#include <KDE/KStandardAction>
#include <KDE/KSystemTrayIcon>

#include <QApplication>
#include <QDesktopServices>
#include <QIcon>
#include <QMenu>
#include <QMessageBox>
#include <QTimer>
#include <QUrl>

MabumbiClient::MabumbiClient()
        : KMainWindow(0), connected(false)
{
    tray = new MabumbiSysTray( this );
    setupActions();
    tray->buildContextMenu();
    tray->show();
    buildMenu();
    initDBusConnection();

    timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), this, SLOT(checkDBusConnection()) );
    timer->start( 5000 );
    timer->singleShot( 200, this, SLOT(startCheck()) );
}

MabumbiClient::~MabumbiClient()
{
}

void MabumbiClient::setupActions()
{
    KActionCollection* actionCollect = tray->actionCollection();
    actionStart = actionCollect->addAction( "Start_Backup", this, SLOT(startBackup()) );
    actionStart->setText( "Start Backup" );
    actionOpen = actionCollect->addAction( "Open_Backup", this, SLOT(openBackup()) );
    actionOpen->setText( "Open Backup" );
}

void MabumbiClient::buildMenu()
{
    KMenuBar* mbar = menuBar();
    QMenu* fileMenu = mbar->addMenu( "&File" );
    KActionCollection* actionCollection = tray->actionCollection();
    QAction* quit = actionCollection->addAction( KStandardAction::Quit, "mainQuit", this, SLOT(quit()) );
    fileMenu->addAction( quit );

    QMenu* aMenu = mbar->addMenu( "&Actions" );
    aMenu->addAction( actionStart );
    aMenu->addAction( actionOpen );

    KMenu* help = helpMenu();
    mbar->addMenu( help );
}

void MabumbiClient::initDBusConnection()
{
    daemon = new OrgMabumbiBackupMabumbiInterface( "org.mabumbi.backup", "/mabumbi",
                           QDBusConnection::systemBus(), this );
    connect( daemon, SIGNAL(startingBackup()), this, SLOT(backupEnteredRunning()) );
    connect( daemon, SIGNAL(idleEntered()), this, SLOT(backupEnteredIdle()) );
    connect( daemon, SIGNAL(error()), this, SLOT(backupError()) );
    connect( daemon, SIGNAL(destinationConnected()), this, SLOT(backupEnteredIdle()) );
    connect( daemon, SIGNAL(destinationDisconnected()), this, SLOT(backupDestinationNotAvailable()) );
}

void MabumbiClient::startBackup()
{
    if ( daemon == 0 )
        return;

    daemon->startBackup();
}

void MabumbiClient::openBackup()
{
    if ( daemon == 0 )
        return;

    QDBusPendingReply<QString> reply = daemon->getBackupDir();
    reply.waitForFinished();
    if( reply.isError() )
    {
        tray->showMessage( reply.error().name(), reply.error().message() );
    }
    else
    {
        QString dir = reply.value();
        QUrl url( dir );
        QDesktopServices::openUrl( url );
    }
}

void MabumbiClient::quit()
{
    QApplication::exit();
}

void MabumbiClient::startCheck()
{
    if ( daemon->isValid() )
    {
        connected = true;
        daemon->checkState();
    }
    else
    {
        connected = false;
        tray->backupError();
        actionStart->setDisabled( true );
        actionOpen->setDisabled( true );
    }
}

void MabumbiClient::checkDBusConnection()
{
    if ( daemon == 0 )
    {
        initDBusConnection();
        return;
    }

    if ( daemon->isValid() )
    {
        if ( !connected )
        {
            connected = true;
            daemon->checkState();
        }
    }
    else
    {
        if ( connected )
        {
            connected = false;
            tray->backupError();
        }
        delete( daemon );
        daemon = 0;
        initDBusConnection();
    }
}

void MabumbiClient::backupEnteredIdle()
{
    tray->backupIdle();
    actionStart->setDisabled( false );
    actionOpen->setDisabled( false );
}

void MabumbiClient::backupEnteredRunning()
{
    tray->backupRunning();
    actionStart->setDisabled( true );
    actionOpen->setDisabled( false );
}

void MabumbiClient::backupDestinationNotAvailable()
{
    tray->destinationDisconnected();
    actionStart->setDisabled( true );
    actionOpen->setDisabled( true );
}

void MabumbiClient::backupError()
{
    tray->backupError();
}

#include "MabumbiClient.moc"
